// Chromatic shader DCI-P3
// By guest.r

// Parameter lines go here:
// Parameter lines go here:
#pragma parameter Xw0 "White Point X"  8946.0  1.0 15000.0 1.0 
#pragma parameter Yw0 "White Point Y" 10000.0  1.0 15000.0 1.0 
#pragma parameter Zw0 "White Point Z"  9544.0  1.0 15000.0 1.0

#pragma parameter xr0 "Red X" 6800.0 1.0 10000.0 1.0
#pragma parameter yr0 "Red Y" 3200.0 1.0 10000.0 1.0

#pragma parameter xg0 "Green X" 2650.0 1.0 10000.0 1.0
#pragma parameter yg0 "Green Y" 6900.0 1.0 10000.0 1.0

#pragma parameter xb0 "Blue X" 1500.0 1.0 10000.0 1.0
#pragma parameter yb0 "Blue Y" 600.0 1.0 10000.0 1.0

#if defined(VERTEX)

#if __VERSION__ >= 130
#define COMPAT_VARYING out
#define COMPAT_ATTRIBUTE in
#define COMPAT_TEXTURE texture
#else
#define COMPAT_VARYING varying 
#define COMPAT_ATTRIBUTE attribute 
#define COMPAT_TEXTURE texture2D
#endif

#ifdef GL_ES
#define COMPAT_PRECISION mediump
#else
#define COMPAT_PRECISION
#endif

COMPAT_ATTRIBUTE vec4 VertexCoord;
COMPAT_ATTRIBUTE vec4 COLOR;
COMPAT_ATTRIBUTE vec4 TexCoord;
COMPAT_VARYING vec4 COL0;
COMPAT_VARYING vec4 TEX0;

uniform mat4 MVPMatrix;
uniform COMPAT_PRECISION int FrameDirection;
uniform COMPAT_PRECISION int FrameCount;
uniform COMPAT_PRECISION vec2 OutputSize;
uniform COMPAT_PRECISION vec2 TextureSize;
uniform COMPAT_PRECISION vec2 InputSize;

void main()
{
    gl_Position = MVPMatrix * VertexCoord;
    COL0 = COLOR;
    TEX0.xy = TexCoord.xy;
}

#elif defined(FRAGMENT)

#if __VERSION__ >= 130
#define COMPAT_VARYING in
#define COMPAT_TEXTURE texture
out vec4 FragColor;
#else
#define COMPAT_VARYING varying
#define FragColor gl_FragColor
#define COMPAT_TEXTURE texture2D
#endif

#ifdef GL_ES
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif
#define COMPAT_PRECISION mediump
#else
#define COMPAT_PRECISION
#endif

uniform COMPAT_PRECISION int FrameDirection;
uniform COMPAT_PRECISION int FrameCount;
uniform COMPAT_PRECISION vec2 OutputSize;
uniform COMPAT_PRECISION vec2 TextureSize;
uniform COMPAT_PRECISION vec2 InputSize;
uniform sampler2D Texture;
uniform sampler2D PassFeedback0;

COMPAT_VARYING vec4 TEX0;
// in variables go here as COMPAT_VARYING whatever

// compatibility #defines
#define Source Texture
#define vTexCoord TEX0.xy

#define SourceSize vec4(TextureSize, 1.0 / TextureSize) //either TextureSize or InputSize
#define outsize vec4(OutputSize, 1.0 / OutputSize)

#ifdef PARAMETER_UNIFORM
// All parameter floats need to have COMPAT_PRECISION in front of them
uniform COMPAT_PRECISION float Xw0;
uniform COMPAT_PRECISION float Yw0;
uniform COMPAT_PRECISION float Zw0;
uniform COMPAT_PRECISION float xr0;
uniform COMPAT_PRECISION float yr0;
uniform COMPAT_PRECISION float xg0;
uniform COMPAT_PRECISION float yg0;
uniform COMPAT_PRECISION float xb0;
uniform COMPAT_PRECISION float yb0;
#else
#define Xw0 1.0
#define Yw0 1.0
#define Zw0 1.0
#define xr0 1.0
#define yr0 1.0
#define xg0 1.0
#define yg0 1.0
#define xb0 1.0
#define yb0 1.0
#endif 

mat3 ToDCI = 
transpose(mat3(
 2.725394,  -1.018003,	-0.440163,
-0.795168,   1.689732,	 0.022647,
 0.041242,	-0.087639,	 1.100929
));

void main()
{
	vec3 c = COMPAT_TEXTURE(Source, TEX0.xy).rgb;

	float Xw = Xw0*0.0001;
	float Yw = Yw0*0.0001;
	float Zw = Zw0*0.0001;
	
	float xr = xr0*0.0001;
	float yr = yr0*0.0001;
	
	float xg = xg0*0.0001;
	float yg = yg0*0.0001;
	
	float xb = xb0*0.0001;
	float yb = yb0*0.0001;
	
	float Xr = xr/yr;
	float Yr = 1.0;
	float Zr = (1.0 - xr - yr)/yr;
	
	float Xg = xg/yg;
	float Yg = 1.0;
	float Zg = (1.0 - xg - yg)/yg;
	
	float Xb = xb/yb;
	float Yb = 1.0;
	float Zb = (1.0 - xb - yb)/yb;

	vec3 S = inverse (transpose (mat3 ( Xr, Xg, Xb,
										Yr, Yg, Yb,
										Zr, Zg, Zb))) * vec3 (Xw, Yw, Zw);
	
	mat3 rgb_xyz = transpose ( mat3 ( S.r*Xr, S.g*Xg, S.b*Xb,
									  S.r*Yr, S.g*Yg, S.b*Yb,
									  S.r*Zr, S.g*Zg, S.b*Zb ));
										
	vec3 c1 = pow(c, vec3(2.60));
	c1 = rgb_xyz * c1;
	c1 = ToAdobe*c1;
	c1 = pow(c1, vec3(1.0/2.60));
	
	FragColor = vec4(c1,1.0);
} 
#endif
